'**********************************************************************
'Program: RadRange.bas. Don Owen 21 Feb 2008
'Uses ATMega8515 micro
'**********************************************************************

'**********************************************************************
'** Description                                                      **
'**********************************************************************

'The purpose of this system is to check the range and reliability of a low
'powered radio link.
'System is comprised of the Tx which has a micro driving a Holtek encoder chip
'HC12E which in turn modulates the RF module ZW3100 Tx (433.92MHz). The Rx
'comprises the RF module ZW3102 Rx (433.92MHz) whose demodulated output
'feeds the Holtek decoder chip HC12D which is in turn fed to the micro.
'This document only deals in detail with the Receive unit.

'This unit processes the Received output via the Holtek decoder chip HC12D
'The Transmitted data burst when successfully decoded produces a momentary high
'level from the decoders VT pin as well as latching the 4 data bits. These are
'fed to the micro. The Rx function synchronises initially with the Tx burst
'which occurs every 5 sec (1/2 sec burst) by placing the Rx & Tx close together
'and then a range check is done by moving them progressively further apart.
'The LCD displays a score of successfull receptions versus unsuccessfull ones.
'The data displayed by the LCD is in decimal form 0 to 15 corresponding to the
'4 data lines .

'The 5 sec timer workings are as follows
'Times/sec  to overflow (TOV) =Xtal freq/(Pscale * 2^16). If xtal is 10000000
'and Pscale is 1024 then TOV = 0.149. which is 6.71 sec.
'Now the time for each count is 6.71/2^16 = 102.4uSec. We need a 5 sec overflow
'so it will take 5/102.4uSec = 48,828 counts to equal 5 sec.
'Therefore Preload = MaxCountOfTimer - 48828
'                  = 65536 - 48828
'                  = 16708
'**********************************************************************
'** Specifications                                                   **
'**********************************************************************

'**********************************************************************
'** Designations                                                     **
'**********************************************************************

'The Port designations are as follows:

'Port A (PA7 to PA0)

'Port B (PB7 to PB0)

'Port C (PC7 to PC0)

'Port D (PD7 to PD0)

'Port E (PE2 to PE0)

'note : LCD used in PIN mode with 4-bit

$regfile = "m8515.dat"
$crystal = 10000000

'**********************************************************************
'** Set up port data                                                 **
'**********************************************************************
'
Config Porta = Input
'Porta is used as input from the Holtek 4 data pins using Pa0 to Pa3
'where Pa0 to Pa3 are used for the 4 data lines and Pa4 is used for the VT sig
Config Pinc.7 = Input                        'VT sig from the Holtek chip
'**********************************************************************
'** Declare subroutines and functions                                **
'**********************************************************************

 '**********************************************************************
'** Define working variables and constants                           **
'**********************************************************************

Dim Rxnum As Integer
Dim Correct As Integer
Dim Incorrect As Integer
Dim First_time As Bit
Dim Data_val As Word
'**********************************************************************
'** Define LCD setup data                                            **
'**********************************************************************

Config Lcd = 16 * 2
Config Lcdpin = Pin , Db4 = Portb.0 , Db5 = Portc.6 , Db6 = Portb.2 , Db7 = Portb.3 , E = Portb.4 , Rs = Portc.2
Cursor Off

'**********************************************************************
'** Actual work starts here.                    .                    **
'**********************************************************************
Cls
Locate 1 , 1
Lcd "Radio Range Unit"
Locate 2 , 1
Lcd "Version 1"
Wait 3
Cls
Locate 1 , 1
Lcd "Waiting on Rx"
Locate 2 , 1
Lcd "to synch to Tx"
Wait 2

Config Timer1 = Timer , Prescale = 1024
Timer1 = 16708                               'preload to interrupt after 5 sec
On Timer1 Timer1_isr
Enable Timer1
Enable Interrupts
Stop Timer1

Set First_time
Rxnum = 0
Correct = 0
Config Debounce = 5
Start_again:
'This loops until a correct signal is received to synch Rx and Tx

Debounce Pinc.7 , 1 , Data_test              'Is VT from coder high?
Goto Start_again

Data_test:
Wait 2
If First_time = 1 Then
  Start Timer1
End If                                       'Valid signal now check data value

Data_val = Pina                              '4 data lines from decoder
Data_val = Data_val And 15                   'Mask top 4 bits
Incr Correct
Goto Start_again
End

'**********************************************************************
'**                        Define ISR                                **
'**********************************************************************

Timer1_isr:
Stop Timer1
Timer1 = 16708                               'preload to interrupt after 5 sec
Start Timer1
Incr Rxnum
If First_time = 1 Then
 Correct = Rxnum
End If
If Correct > Rxnum Then
 Incorrect = 0
End If
Incorrect = Rxnum - Correct
Cls
Locate 1 , 1
Lcd "Test " ; Rxnum ; " Dat " ; Data_val
Locate 2 , 1
Lcd "Good " ; Correct ; " Bad " ; Incorrect
'Tx_off:
Reset First_time

Return